### Go 大杀器之跟踪剖析 trace
https://juejin.im/post/5d27400151882530af139a85

### pprof 定位oom
https://yq.aliyun.com/articles/573743
https://studygolang.com/articles/20519

### Go 程序的性能调试问题
https://studygolang.com/articles/2498 
### Debugging performance issues in Go programs
https://software.intel.com/zh-cn/blogs/2014/05/10/debugging-performance-issues-in-go-programs

### Go, the unwritten parts
https://rakyll.org/

// $GODEV/bin/go 使用开发模式
// Gosched 让出cpu
// Goexit 退出当前goroutine(但是defer语句会照常执行)
// GOMAXPROCS  设置逻辑处理器数量，即最大的可同时使用的CPU核数 set env: export GOMAXPROCS=1
// NumCPU 返回当前系统的CPU核数量

##  build
`go build -o main -gcflags "all=-N -l"`

### 查看go tool 命令详细信息
`go tool compile -d help`
usage: -d arg[,arg]* and arg is <key>[=<value>]

<key> is one of:

	append       	print information about append compilation
	closure      	print information about closure compilation
	compilelater 	compile functions as late as possible
	disablenil   	disable nil checks
	dclstack     	run internal dclstack check
	gcprog       	print dump of GC programs
	nil          	print information about nil checks
	panic        	do not hide any compiler panic
	slice        	print information about slice compilation
	typeassert   	print information about type assertion inlining
	wb           	print information about write barriers
	export       	print export data
	pctab        	print named pc-value table
	locationlists	print information about DWARF location list creation
	typecheckinl 	eager typechecking of inline function bodies
	dwarfinl     	print information about DWARF inlined function creation
	softfloat    	force compiler to emit soft-float code
	ssa/help     	print help about SSA debugging

<value> is key-specific.

Key "pctab" supports values:
	"pctospadj", "pctofile", "pctoline", "pctoinline", "pctopcdata"

### 打印ssa编译 相关信息
  `go tool compile -d=ssa/help`
  `go tool compile -d=ssa/<phase>/<flag>[=<value>|<function_name>]`

  `go tool compile -d=slice,append,gcprog,closure,export,wb -o t.a main.go`

  `go tool compile -d=ssa/stackframe/on,ssa/stackframe/dump main.go`
  `go tool compile -d=ssa/check/on,ssa/all/time main.go`

  `go tool compile -d=ssa/prove/debug=2 main.go`
  main.go:10:12: x+d >= w; x:v29 b1 delta:1 w:0 d:signed

  `go tool compile -d=ssa/stackframe/time main.go`
  <autogenerated>:1:      stackframe      TIME(ns)        2428    init
  main.go:10:12:  stackframe      TIME(ns)        14757   main
  main.go:16:9:   stackframe      TIME(ns)        5389    adder
  main.go:20:3:   stackframe      TIME(ns)        5034    adder.func1

  `go tool compile -d=ssa/stackframe/mem main.go` 
  <autogenerated>:1:      stackframe      TIME(ns):BYTES:ALLOCS   1370    0       0       init
  main.go:10:12:  stackframe      TIME(ns):BYTES:ALLOCS   18849   32      1       main
  main.go:16:9:   stackframe      TIME(ns):BYTES:ALLOCS   4500    32      1       adder
  main.go:20:3:   stackframe      TIME(ns):BYTES:ALLOCS   4125    32      1       adder.func1

  `go tool compile -d=ssa/schedule/mem main.go` 
  <autogenerated>:1:      schedule        TIME(ns):BYTES:ALLOCS   7627    952     7       init
  main.go:10:12:  schedule        TIME(ns):BYTES:ALLOCS   39538   4832    10      main
  main.go:16:9:   schedule        TIME(ns):BYTES:ALLOCS   17515   1304    10      adder
  main.go:20:3:   schedule        TIME(ns):BYTES:ALLOCS   9290    1096    10      adder.func1

### 打印所有ssa生成阶段代码，并且生成交互式ssa.html
  `GOSSAFUNC=main GOOS=linux GOARCH=amd64 go build -gcflags -S main.go > ssa.html` 

## print asm

### 查看xxx包所有生成的init函数
`go tool objdump -s "runtime\.init\b" main`
`go tool objdump -s "main\.init\b" main`
`go tool objdump -s "http\.init\b" main | grep "init"`

```cmd
# go tool objdump -s "main\.init\b" main
TEXT main.init(SB) <autogenerated>
  <autogenerated>:1	0x487420		64488b0c25f8ffffff	MOVQ FS:0xfffffff8, CX			
  <autogenerated>:1	0x487429		483b6110		CMPQ 0x10(CX), SP			
  <autogenerated>:1	0x48742d		764e			JBE 0x48747d				
  <autogenerated>:1	0x48742f		4883ec08		SUBQ $0x8, SP				
  <autogenerated>:1	0x487433		48892c24		MOVQ BP, 0(SP)				
  <autogenerated>:1	0x487437		488d2c24		LEAQ 0(SP), BP				
  <autogenerated>:1	0x48743b		803dc5ba0e0001		CMPB $0x1, main.initdone.(SB)		
  <autogenerated>:1	0x487442		7702			JA 0x487446				
  <autogenerated>:1	0x487444		eb09			JMP 0x48744f				
  <autogenerated>:1	0x487446		488b2c24		MOVQ 0(SP), BP				
  <autogenerated>:1	0x48744a		4883c408		ADDQ $0x8, SP				
  <autogenerated>:1	0x48744e		c3			RET					
  <autogenerated>:1	0x48744f		803db1ba0e0001		CMPB $0x1, main.initdone.(SB)		
  <autogenerated>:1	0x487456		7402			JE 0x48745a				
  <autogenerated>:1	0x487458		eb07			JMP 0x487461				
  <autogenerated>:1	0x48745a		e861f7f9ff		CALL runtime.throwinit(SB)		
  <autogenerated>:1	0x48745f		0f0b			UD2					
  <autogenerated>:1	0x487461		c6059fba0e0001		MOVB $0x1, main.initdone.(SB)		
  <autogenerated>:1	0x487468		e8e3fbffff		CALL fmt.init(SB)			
  <autogenerated>:1	0x48746d		c60593ba0e0002		MOVB $0x2, main.initdone.(SB)		
  <autogenerated>:1	0x487474		488b2c24		MOVQ 0(SP), BP				
  <autogenerated>:1	0x487478		4883c408		ADDQ $0x8, SP				
  <autogenerated>:1	0x48747c		c3			RET					
  <autogenerated>:1	0x48747d		e87e7efcff		CALL runtime.morestack_noctxt(SB)	
  <autogenerated>:1	0x487482		eb9c			JMP main.init(SB)	

```

### 查看源代码及其对应的汇编
`go tool objdump -S -s "main\.main" main`

`go tool compile -S -N -l main.go | grep "main.go:3"`

`go tool compile -N -l -S main.go > main.s`

`go build -gcflags -S main.go  more asm`

prints the disassembly only for package fmt
`go build -gcflags=-S fmt`
prints the disassembly for fmt and all its dependencies
`go build -gcflags=all=-S fmt`

## gctrace
`go build -o main -gcflags "-N -l" && GODEBUG=gctrace=1   ./main `
`go build -o main /data/app/go/src/gosrc/demo/runtime/goroutine/for/main.go && GODEBUG=gctrace=1,gccheckmark=0  ./main`

##  goroutine schedtrace
`go build -o main demo/runtime/goroutine/main.go && GODEBUG=schedtrace=10000,scheddetail=1 ./main`


`sudo go tool dist test -v -run=^go_test:runtime$`

## test
``go tool compile -N -l -S demo/type/interface/struct/main.go > main.s`

`go build -o main -gcflags "all=-N -l" demo/type/interface/struct/main.go && go tool objdump -S -s "main\.main" main > main.s`

### list release tag
`go list  -f '{{context.ReleaseTags}}' runtime | grep go1.10`
 [go1.1 go1.2 go1.3 go1.4 go1.5 go1.6 go1.7 go1.8 go1.9 go1.10 go1.11]

`go build -o main -gcflags "all=-N -l -m" demo/runtime/memory/closure/main.go`

### go doc 
`go doc -all -u runtime.slice`
`go doc -all -u reflect.SliceHeader`

### Visualise Go program GC trace data in real time 
https://github.com/davecheney/gcvis

`allocfreetrace: 设置 allocfreetrace=1 会监控每次分配，但因每次分配和释放的栈信息（stack trace）`

`cgocheck: 设置 cgocheck=0 禁用所有cgo检查将Go指针传递给非Go代码是否正确。cgocheck=1 (缺省值) 轻量级检查。cgocheck=2 重量级检查。`

`efence: 设置 efence=1 导致分配器 allocator将每个对象分配在一个唯一的页page上，地址不重用。`

`gccheckmark: 设置 gccheckmark=1 允许垃圾回收器执行并发mark阶段的校验。会导致Stop The World。`

`gcpacertrace: 设置 gcpacertrace=1 会让来几回收器打印出concurrent pacer的内部状态。`

`gcshrinkstackoff: 设置 gcshrinkstackoff=1 则禁止将 goroutines 的栈缩小为更小栈。`

`gcstackbarrieroff: 设置 gcstackbarrieroff=1 禁用stack barriers，会影响垃圾回收器的重复搜索栈的功能。`

`gcstackbarrierall: 设置 gcstackbarrierall=1 会为每个栈帧安装一 stack barriers。`

`gcstoptheworld: 设置 gcstoptheworld=1 则禁用并发垃圾回收,每次回收都会触发STW。设置gcstoptheworld=2则禁用垃圾回收后的concurrent sweeping。`

`gctrace: 设置 gctrace=1导致每次垃圾回收器触发一行日志，包含内存回收的概要信息和暂停的时间。设置gctrace=2起同样的效果，but also repeats each collection。格式如下：`

    gc # @#s #%: #+#+# ms clock, #+#/#/#+# ms cpu, #->#-># MB, # MB goal, # P
where the fields are as follows:
    gc #        GC id,每次GC加一
    @#s         程序启动后的时间，单位秒
    #%          程序启动后GC所用的时间比
    #+...+#     此次GC所用的wall-clock/CPU时间
    #->#-># MB  GC开始时的堆大小, GC结束时的堆大小, 活着的(live)堆大小
    # MB goal   总的堆大小
    # P         CPU使用数
垃圾回收分为下面的几个阶段：stop-the-world (STW) sweep termination, concurrent
mark and scan, and STW mark termination。 mark/scan的CPU时间又分为 assist time (GC performed in
line with allocation), background GC time, and idle GC time。
如果日志后以"(forced)"结尾,则GC通过runtime.GC()调用执行，此时所有的阶段都是STW.

`memprofilerate: 设置 memprofilerate=X 会更新runtime.MemProfileRate的值。0则禁用这个profie。`

`invalidptr: 默认设为invalidptr=1, 如果指针被赋予一个无效值,会引起程序的崩溃，设置该值为0，会停止该检查，0只能临时用于查找bug，真正的解决方法是不要把整数类型的值存在指针变量里面。`

`sbrk: 设置 sbrk=1 会使用实验性的实现替换memory allocator 和 garbage collector。`

`scavenge: scavenge=1 允许heap scavenger的debug模式。`

`scheddetail: 设置 schedtrace=X 和 scheddetail=1 会导致goroutine调度器每个X毫秒输出多行调度信息。`

`schedtrace: 设置 schedtrace=X导致调度器每个X秒输出一行调度器的概要信息。`

```