package main

import (
	"crypto/md5"
	"errors"
	"fmt"
	"io/ioutil"
	"os"
	"path/filepath"
	"sort"
	"sync"
)

type result struct {
	path string
	sum  [md5.Size]byte
	err  error
}

func main() {
	// 计算指定目录下所有文件的MD5值，之后按照目录名排序并打印结果
	m, err := MD5All3(os.Args[1])
	if err != nil {
		fmt.Println(err)
		return
	}
	var paths []string
	for path := range m {
		paths = append(paths, path)
	}
	sort.Strings(paths)
	// fmt.Println(m)
	for _, path := range paths {
		fmt.Printf("path:%s md5:%x\n",  path, m[path])
	}
}

func walkFiles(done <-chan struct{}, root string) (<-chan string, <-chan error) {
	paths := make(chan string)
	errc := make(chan error, 1)
	go func() {
		// 在Walk之后关闭paths channel
		defer close(paths)
		// 因为errc有缓冲区，所以这里不需要select。
		errc <- filepath.Walk(root, func(path string, info os.FileInfo, err error) error {
			if err != nil {
				return err
			}
			if info.IsDir() {
				return nil
			}
			select {
			case paths <- path:
			case <-done:
				return errors.New("walk canceled")
			}
			return nil
		})
	}()
	return paths, errc
}

func digester(done <-chan struct{}, paths <-chan string, c chan<- result) {
	for path := range paths {
		data, err := ioutil.ReadFile(path)
		select {
		case c <- result{path, md5.Sum(data), err}:
		case <-done:
			return
		}
	}
}

// MD5All 在返回时关闭done channel；这个可能在从c和errc收到所有的值之前被调用
func MD5All3(root string) (m map[string][md5.Size]byte, err error) {
	// 启动固定数量的Goroutine来读取并对文件做摘要。
	c := make(chan result)
	done := make(chan struct{})
	defer close(done)
	var wg sync.WaitGroup
	const numDigesters = 5
	paths, errc := walkFiles(done, root)
	wg.Add(numDigesters)
	for i := 0; i < numDigesters; i++ {
		go func() {
			digester(done, paths, c)
			wg.Done()
		}()
	}
	go func() {
		wg.Wait()
		close(c)
	}()

	m = make(map[string][md5.Size]byte)
	for r := range c {
		if r.err != nil {
			return nil, r.err
		}
		// fmt.Println(r.sum)//byte array
		m[r.path] = r.sum
	}
	// 检查Walk是否失败
	if err = <-errc; err != nil {
		return nil, err
	}
	return m, nil
}
